from .task import Task as Task, TaskQueue as TaskQueue
from _typeshed import Incomplete

class CancelledError(BaseException): ...
class TimeoutError(Exception): ...

_exc_context: Incomplete

class SingletonGenerator:
    state: Incomplete
    exc: Incomplete
    def __init__(self) -> None: ...
    def __iter__(self): ...
    def __next__(self) -> None: ...

def sleep_ms(t, sgen=...): ...
def sleep(t): ...

class IOQueue:
    poller: Incomplete
    map: Incomplete
    def __init__(self) -> None: ...
    def _enqueue(self, s, idx) -> None: ...
    def _dequeue(self, s) -> None: ...
    def queue_read(self, s) -> None: ...
    def queue_write(self, s) -> None: ...
    def remove(self, task) -> None: ...
    def wait_io_event(self, dt) -> None: ...

def _promote_to_task(aw): ...
def create_task(coro): ...
def run_until_complete(main_task: Incomplete | None = ...): ...
def run(coro): ...
async def _stopper() -> None: ...

_stop_task: Incomplete

class Loop:
    _exc_handler: Incomplete
    def create_task(coro): ...
    def run_forever() -> None: ...
    def run_until_complete(aw): ...
    def stop() -> None: ...
    def close() -> None: ...
    def set_exception_handler(handler) -> None: ...
    def get_exception_handler(): ...
    def default_exception_handler(loop, context) -> None: ...
    def call_exception_handler(context) -> None: ...

def get_event_loop(runq_len: int = ..., waitq_len: int = ...): ...
def current_task(): ...
def new_event_loop(): ...



from _typeshed import Incomplete
from ubinascii import hexlify as hexlify

class MQTTException(Exception): ...

class MQTTClient:
    client_id: Incomplete
    sock: Incomplete
    server: Incomplete
    port: Incomplete
    ssl: Incomplete
    ssl_params: Incomplete
    pid: int
    cb: Incomplete
    user: Incomplete
    pswd: Incomplete
    keepalive: Incomplete
    lw_topic: Incomplete
    lw_msg: Incomplete
    lw_qos: int
    lw_retain: bool
    def __init__(
        self,
        client_id,
        server,
        port: int = ...,
        user: Incomplete | None = ...,
        password: Incomplete | None = ...,
        keepalive: int = ...,
        ssl: bool = ...,
        ssl_params=...,
    ) -> None: ...
    def _send_str(self, s) -> None: ...
    def _recv_len(self): ...
    def set_callback(self, f) -> None: ...
    def set_last_will(self, topic, msg, retain: bool = ..., qos: int = ...) -> None: ...
    def connect(self, clean_session: bool = ...): ...
    def disconnect(self) -> None: ...
    def ping(self) -> None: ...
    def publish(self, topic, msg, retain: bool = ..., qos: int = ...) -> None: ...
    def subscribe(self, topic, qos: int = ...) -> None: ...
    def wait_msg(self): ...
    def check_msg(self): ...



from _typeshed import Incomplete

class Response:
    raw: Incomplete
    encoding: str
    _cached: Incomplete
    def __init__(self, f) -> None: ...
    def close(self) -> None: ...
    @property
    def content(self): ...
    @property
    def text(self): ...
    def json(self): ...

def request(
    method,
    url,
    data: Incomplete | None = ...,
    json: Incomplete | None = ...,
    headers=...,
    stream: Incomplete | None = ...,
    auth: Incomplete | None = ...,
    timeout: Incomplete | None = ...,
    parse_headers: bool = ...,
): ...
def head(url, **kw): ...
def get(url, **kw): ...
def post(url, **kw): ...
def put(url, **kw): ...
def patch(url, **kw): ...
def delete(url, **kw): ...



"""
Control of WS2812 / NeoPixel LEDs.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/neopixel.html

This module provides a driver for WS2818 / NeoPixel LEDs.

``Note:`` This module is only included by default on the ESP8266, ESP32 and RP2
   ports. On STM32 / Pyboard and others, you can either install the
   ``neopixel`` package using :term:`mip`, or you can download the module
   directly from
   <https://raw.githubusercontent.com/micropython/micropython-lib/master/micropython/drivers/led/neopixel/neopixel.py>`_
   and copy it to the filesystem.
"""
from _typeshed import Incomplete
from typing import Tuple

class NeoPixel:
    """
    Construct an NeoPixel object.  The parameters are:

        - *pin* is a machine.Pin instance.
        - *n* is the number of LEDs in the strip.
        - *bpp* is 3 for RGB LEDs, and 4 for RGBW LEDs.
        - *timing* is 0 for 400KHz, and 1 for 800kHz LEDs (most are 800kHz).
    """

    ORDER: Incomplete
    pin: Incomplete
    n: Incomplete
    bpp: Incomplete
    buf: Incomplete
    timing: Incomplete
    def __init__(self, pin, n, *, bpp=3, timing=1) -> None: ...
    def __len__(self) -> int:
        """
        Returns the number of LEDs in the strip.
        """
        ...
    def __setitem__(self, index, val) -> None:
        """
        Set the pixel at *index* to the value, which is an RGB/RGBW tuple.
        """
        ...
    def __getitem__(self, index) -> Tuple:
        """
        Returns the pixel at *index* as an RGB/RGBW tuple.
        """
        ...
    def fill(self, pixel) -> None:
        """
        Sets the value of all pixels to the specified *pixel* value (i.e. an
        RGB/RGBW tuple).
        """
        ...
    def write(self) -> None:
        """
        Writes the current pixel data to the strip.
        """
        ...



"""
Cryptographic ciphers.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/cryptolib.html
"""
from typing import Optional, Any
from _typeshed import Incomplete

class aes:
    def encrypt(self, in_buf, out_buf: Optional[Any] = None) -> Incomplete:
        """
        Encrypt *in_buf*. If no *out_buf* is given result is returned as a
        newly allocated `bytes` object. Otherwise, result is written into
        mutable buffer *out_buf*. *in_buf* and *out_buf* can also refer
        to the same mutable buffer, in which case data is encrypted in-place.
        """
        ...
    def decrypt(self, in_buf, out_buf: Optional[Any] = None) -> Incomplete:
        """
        Like `encrypt()`, but for decryption.
        """
        ...
    def __init__(self, key, mode, IV: Optional[Any] = None) -> None:
        """
        Initialize cipher object, suitable for encryption/decryption. Note:
        after initialization, cipher object can be use only either for
        encryption or decryption. Running decrypt() operation after encrypt()
        or vice versa is not supported.

        Parameters are:

            * *key* is an encryption/decryption key (bytes-like).
            * *mode* is:

                * ``1`` (or ``cryptolib.MODE_ECB`` if it exists) for Electronic Code Book (ECB).
                * ``2`` (or ``cryptolib.MODE_CBC`` if it exists) for Cipher Block Chaining (CBC).
                * ``6`` (or ``cryptolib.MODE_CTR`` if it exists) for Counter mode (CTR).

            * *IV* is an initialization vector for CBC mode.
            * For Counter mode, *IV* is the initial value for the counter.
        """
        ...



import os
from _typeshed import Incomplete

class LS:
    def __repr__(self) -> str: ...
    def __call__(self, path: str = ...) -> None: ...

class PWD:
    def __repr__(self) -> str: ...
    def __call__(self): ...

class CLEAR:
    def __repr__(self) -> str: ...
    def __call__(self): ...

def head(f, n: int = ...) -> None: ...
def cat(f) -> None: ...
def cp(s, t) -> None: ...
def newfile(path) -> None: ...
def rm(d, recursive: bool = ...) -> None: ...

class Man:
    def __repr__(self) -> str: ...

man: Incomplete
pwd: Incomplete
ls: Incomplete
clear: Incomplete
cd = os.chdir
mkdir = os.mkdir
mv = os.rename
rmdir = os.rmdir



from typing import Any

def reset(*args, **kwargs) -> Any: ...
def writebyte(*args, **kwargs) -> Any: ...
def writebit(*args, **kwargs) -> Any: ...
def crc8(*args, **kwargs) -> Any: ...
def readbyte(*args, **kwargs) -> Any: ...
def readbit(*args, **kwargs) -> Any: ...



"""
Zlib decompression.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/zlib.html

CPython module: :mod:`python:zlib` https://docs.python.org/3/library/zlib.html .

This module allows to decompress binary data compressed with
`DEFLATE algorithm <https://en.wikipedia.org/wiki/DEFLATE>`_
(commonly used in zlib library and gzip archiver). Compression
is not yet implemented.
"""
from typing import Any
from _typeshed import Incomplete

def decompress(data, wbits=0, bufsize=0, /) -> bytes:
    """
    Return decompressed *data* as bytes. *wbits* is DEFLATE dictionary window
    size used during compression (8-15, the dictionary size is power of 2 of
    that value). Additionally, if value is positive, *data* is assumed to be
    zlib stream (with zlib header). Otherwise, if it's negative, it's assumed
    to be raw DEFLATE stream. *bufsize* parameter is for compatibility with
    CPython and is ignored.
    """
    ...

class DecompIO:
    """
    Create a `stream` wrapper which allows transparent decompression of
    compressed data in another *stream*. This allows to process compressed
    streams with data larger than available heap size. In addition to
    values described in :func:`decompress`, *wbits* may take values
    24..31 (16 + 8..15), meaning that input stream has gzip header.

    Difference to CPython

       This class is MicroPython extension. It's included on provisional
       basis and may be changed considerably or removed in later versions.
    """

    def readinto(self, *args, **kwargs) -> Any: ...
    def readline(self, *args, **kwargs) -> Any: ...
    def read(self, *args, **kwargs) -> Any: ...
    def __init__(self, stream, wbits=0, /) -> None: ...



"""
Control the garbage collector.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/gc.html

CPython module: :mod:`python:gc` https://docs.python.org/3/library/gc.html .
"""
from typing import Optional, Any
from _typeshed import Incomplete

def mem_alloc() -> int:
    """
    Return the number of bytes of heap RAM that are allocated.

    Difference to CPython

       This function is MicroPython extension.
    """
    ...

def isenabled(*args, **kwargs) -> Any: ...
def mem_free() -> int:
    """
    Return the number of bytes of available heap RAM, or -1 if this amount
    is not known.

    Difference to CPython

       This function is MicroPython extension.
    """
    ...

def threshold(amount: Optional[Any] = None) -> Incomplete:
    """
    Set or query the additional GC allocation threshold. Normally, a collection
    is triggered only when a new allocation cannot be satisfied, i.e. on an
    out-of-memory (OOM) condition. If this function is called, in addition to
    OOM, a collection will be triggered each time after *amount* bytes have been
    allocated (in total, since the previous time such an amount of bytes
    have been allocated). *amount* is usually specified as less than the
    full heap size, with the intention to trigger a collection earlier than when the
    heap becomes exhausted, and in the hope that an early collection will prevent
    excessive memory fragmentation. This is a heuristic measure, the effect
    of which will vary from application to application, as well as
    the optimal value of the *amount* parameter.

    Calling the function without argument will return the current value of
    the threshold. A value of -1 means a disabled allocation threshold.

    Difference to CPython

       This function is a MicroPython extension. CPython has a similar
       function - ``set_threshold()``, but due to different GC
       implementations, its signature and semantics are different.
    """
    ...

def collect() -> None:
    """
    Run a garbage collection.
    """
    ...

def enable() -> None:
    """
    Enable automatic garbage collection.
    """
    ...

def disable() -> None:
    """
    Disable automatic garbage collection.  Heap memory can still be allocated,
    and garbage collection can still be initiated manually using :meth:`gc.collect`.
    """
    ...



"""
System specific functions.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/sys.html

CPython module: :mod:`python:sys` https://docs.python.org/3/library/sys.html .
"""
from typing import Dict, List, Tuple, Any
from _typeshed import Incomplete

platform: str
version_info: tuple
path: list
version: str
ps1: str
ps2: str
byteorder: str
modules: dict
argv: list
implementation: tuple
maxsize: int

def print_exception(exc, file=stdout, /) -> None:
    """
    Print exception with a traceback to a file-like object *file* (or
    `sys.stdout` by default).

    Difference to CPython

       This is simplified version of a function which appears in the
       ``traceback`` module in CPython. Unlike ``traceback.print_exception()``,
       this function takes just exception value instead of exception type,
       exception value, and traceback object; *file* argument should be
       positional; further arguments are not supported. CPython-compatible
       ``traceback`` module can be found in `micropython-lib`.
    """
    ...

def exit(retval=0, /) -> Incomplete:
    """
    Terminate current program with a given exit code. Underlyingly, this
    function raise as `SystemExit` exception. If an argument is given, its
    value given as an argument to `SystemExit`.
    """
    ...

stderr: Any
stdout: Any
stdin: Any



from _typeshed import Incomplete

class OneWireError(Exception): ...

class OneWire:
    SEARCH_ROM: int
    MATCH_ROM: int
    SKIP_ROM: int
    pin: Incomplete
    def __init__(self, pin) -> None: ...
    def reset(self, required: bool = ...): ...
    def readbit(self): ...
    def readbyte(self): ...
    def readinto(self, buf) -> None: ...
    def writebit(self, value): ...
    def writebyte(self, value): ...
    def write(self, buf) -> None: ...
    def select_rom(self, rom) -> None: ...
    def scan(self): ...
    def _search_rom(self, l_rom, diff): ...
    def crc8(self, data): ...



from . import core as core
from _typeshed import Incomplete
from collections.abc import Generator

async def _run(waiter, aw) -> None: ...
async def wait_for(aw, timeout, sleep=...): ...
def wait_for_ms(aw, timeout): ...

class _Remove:
    @staticmethod
    def remove(t) -> None: ...

def gather(*aws, return_exceptions: bool = ...) -> Generator[None, None, Incomplete]: ...



RC: str
CONFIG: str

def input_choice(prompt, choices): ...
def getpass(prompt): ...
def input_pass(): ...
def exists(fname): ...
def get_daemon_status(): ...
def change_daemon(action) -> None: ...
def main() -> None: ...



import uio
from . import core as core
from _typeshed import Incomplete
from collections.abc import Generator

class Event:
    state: bool
    waiting: Incomplete
    def __init__(self) -> None: ...
    def is_set(self): ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    def wait(self) -> Generator[None, None, Incomplete]: ...

class ThreadSafeFlag(uio.IOBase):
    state: int
    def __init__(self) -> None: ...
    def ioctl(self, req, flags): ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    async def wait(self) -> Generator[Incomplete, None, None]: ...



from . import core as core
from _typeshed import Incomplete
from collections.abc import Generator

class Lock:
    state: int
    waiting: Incomplete
    def __init__(self) -> None: ...
    def locked(self): ...
    def release(self) -> None: ...
    def acquire(self) -> Generator[None, None, Incomplete]: ...
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc, tb): ...



"""
Network configuration.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/network.html

This module provides network drivers and routing configuration. To use this
module, a MicroPython variant/build with network capabilities must be installed.
Network drivers for specific hardware are available within this module and are
used to configure hardware network interface(s). Network services provided
by configured interfaces are then available for use via the :mod:`socket`
module.

For example::

    # connect/ show IP config a specific network interface
    # see below for examples of specific drivers
    import network
    import time
    nic = network.Driver(...)
    if not nic.isconnected():
        nic.connect()
        print("Waiting for connection...")
        while not nic.isconnected():
            time.sleep(1)
    print(nic.ifconfig())

    # now use socket as usual
    import socket
    addr = socket.getaddrinfo('micropython.org', 80)[0][-1]
    s = socket.socket()
    s.connect(addr)
    s.send(b'GET / HTTP/1.1\r\nHost: micropython.org\r\n\r\n')
    data = s.recv(1000)
    s.close()
"""
from typing import List, Optional, Tuple, Union, Any
from _typeshed import Incomplete

STA_IF: int
STAT_WRONG_PASSWORD: int
STAT_ASSOC_FAIL: int
MODE_11G: int
MODE_LR: int
MODE_11N: int
STAT_IDLE: int
STAT_BEACON_TIMEOUT: int
STAT_NO_AP_FOUND: int
STAT_CONNECTING: int
STAT_HANDSHAKE_TIMEOUT: int
STAT_GOT_IP: int
AUTH_WAPI_PSK: int
MODE_11B: int
AUTH_WEP: int
AP_IF: int
AUTH_OPEN: int
AUTH_MAX: int
AUTH_WPA_PSK: int
AUTH_WPA2_ENTERPRISE: int
AUTH_WPA_WPA2_PSK: int
AUTH_WPA2_PSK: int
AUTH_WPA3_PSK: int
AUTH_WPA2_WPA3_PSK: int

def country(*args, **kwargs) -> Any: ...
def hostname(*args, **kwargs) -> Any: ...
def phy_mode(*args, **kwargs) -> Any: ...

class WLAN:
    """
    Create a WLAN network interface object. Supported interfaces are
    ``network.STA_IF`` (station aka client, connects to upstream WiFi access
    points) and ``network.AP_IF`` (access point, allows other WiFi clients to
    connect). Availability of the methods below depends on interface type.
    For example, only STA interface may `WLAN.connect()` to an access point.
    """

    def __init__(self, interface_id) -> None: ...
    def active(self, is_active: Optional[Any] = None) -> None:
        """
        Activate ("up") or deactivate ("down") network interface, if boolean
        argument is passed. Otherwise, query current state if no argument is
        provided. Most other methods require active interface.
        """
        ...
    def connect(self, ssid=None, key=None, *, bssid=None) -> None:
        """
        Connect to the specified wireless network, using the specified key.
        If *bssid* is given then the connection will be restricted to the
        access-point with that MAC address (the *ssid* must also be specified
        in this case).
        """
        ...
    def disconnect(self) -> None:
        """
        Disconnect from the currently connected wireless network.
        """
        ...
    def scan(self) -> List[Tuple]:
        """
        Scan for the available wireless networks.
        Hidden networks -- where the SSID is not broadcast -- will also be scanned
        if the WLAN interface allows it.

        Scanning is only possible on STA interface. Returns list of tuples with
        the information about WiFi access points:

            (ssid, bssid, channel, RSSI, security, hidden)

        *bssid* is hardware address of an access point, in binary form, returned as
        bytes object. You can use `binascii.hexlify()` to convert it to ASCII form.

        There are five values for security:

            * 0 -- open
            * 1 -- WEP
            * 2 -- WPA-PSK
            * 3 -- WPA2-PSK
            * 4 -- WPA/WPA2-PSK

        and two for hidden:

            * 0 -- visible
            * 1 -- hidden
        """
        ...
    def status(self, param: Optional[Any] = None) -> Incomplete:
        """
        Return the current status of the wireless connection.

        When called with no argument the return value describes the network link status.
        The possible statuses are defined as constants:

            * ``STAT_IDLE`` -- no connection and no activity,
            * ``STAT_CONNECTING`` -- connecting in progress,
            * ``STAT_WRONG_PASSWORD`` -- failed due to incorrect password,
            * ``STAT_NO_AP_FOUND`` -- failed because no access point replied,
            * ``STAT_CONNECT_FAIL`` -- failed due to other problems,
            * ``STAT_GOT_IP`` -- connection successful.

        When called with one argument *param* should be a string naming the status
        parameter to retrieve.  Supported parameters in WiFI STA mode are: ``'rssi'``.
        """
        ...
    def isconnected(self) -> bool:
        """
        In case of STA mode, returns ``True`` if connected to a WiFi access
        point and has a valid IP address.  In AP mode returns ``True`` when a
        station is connected. Returns ``False`` otherwise.
        """
        ...
    def ifconfig(self, configtuple: Optional[Any] = None) -> Tuple:
        """
        Get/set IP-level network interface parameters: IP address, subnet mask,
        gateway and DNS server. When called with no arguments, this method returns
        a 4-tuple with the above information. To set the above values, pass a
        4-tuple with the required information.  For example::

         nic.ifconfig(('192.168.0.4', '255.255.255.0', '192.168.0.1', '8.8.8.8'))
        """
        ...
    def config(self, *args, **kwargs) -> Incomplete:
        """
        Get or set general network interface parameters. These methods allow to work
        with additional parameters beyond standard IP configuration (as dealt with by
        `WLAN.ifconfig()`). These include network-specific and hardware-specific
        parameters. For setting parameters, keyword argument syntax should be used,
        multiple parameters can be set at once. For querying, parameters name should
        be quoted as a string, and only one parameter can be queries at time::

         # Set WiFi access point name (formally known as SSID) and WiFi channel
         ap.config(ssid='My AP', channel=11)
         # Query params one by one
         print(ap.config('ssid'))
         print(ap.config('channel'))

        Following are commonly supported parameters (availability of a specific parameter
        depends on network technology type, driver, and :term:`MicroPython port`).

        =============  ===========
        Parameter      Description
        =============  ===========
        mac            MAC address (bytes)
        ssid           WiFi access point name (string)
        channel        WiFi channel (integer)
        hidden         Whether SSID is hidden (boolean)
        security       Security protocol supported (enumeration, see module constants)
        key            Access key (string)
        hostname       The hostname that will be sent to DHCP (STA interfaces) and mDNS (if supported, both STA and AP). (Deprecated, use :func:`network.hostname` instead)
        reconnects     Number of reconnect attempts to make (integer, 0=none, -1=unlimited)
        txpower        Maximum transmit power in dBm (integer or float)
        =============  ===========
        """
        ...

def PPP(*args, **kwargs) -> Any: ...



from typing import Any

class TaskQueue:
    def push(self, *args, **kwargs) -> Any: ...
    def peek(self, *args, **kwargs) -> Any: ...
    def remove(self, *args, **kwargs) -> Any: ...
    def pop(self, *args, **kwargs) -> Any: ...
    def __init__(self, *argv, **kwargs) -> None: ...

class Task:
    def __init__(self, *argv, **kwargs) -> None: ...



from . import core as core
from _typeshed import Incomplete

def ph_meld(h1, h2): ...
def ph_pairing(child): ...
def ph_delete(heap, node): ...

class TaskQueue:
    heap: Incomplete
    def __init__(self) -> None: ...
    def peek(self): ...
    def push(self, v, key: Incomplete | None = ...) -> None: ...
    def pop(self): ...
    def remove(self, v) -> None: ...

class Task:
    coro: Incomplete
    data: Incomplete
    state: bool
    ph_key: int
    ph_child: Incomplete
    ph_child_last: Incomplete
    ph_next: Incomplete
    ph_rightmost_parent: Incomplete
    def __init__(self, coro, globals: Incomplete | None = ...) -> None: ...
    def __iter__(self): ...
    def __next__(self) -> None: ...
    def done(self): ...
    def cancel(self): ...



from _typeshed import Incomplete

class DHTBase:
    pin: Incomplete
    buf: Incomplete
    def __init__(self, pin) -> None: ...
    def measure(self) -> None: ...

class DHT11(DHTBase):
    def humidity(self): ...
    def temperature(self): ...

class DHT22(DHTBase):
    def humidity(self): ...
    def temperature(self): ...



"""
Access binary data in a structured way.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/uctypes.html

This module implements "foreign data interface" for MicroPython. The idea
behind it is similar to CPython's ``ctypes`` modules, but the actual API is
different, streamlined and optimized for small size. The basic idea of the
module is to define data structure layout with about the same power as the
C language allows, and then access it using familiar dot-syntax to reference
sub-fields.
"""
from typing import Any
from _typeshed import Incomplete

VOID: int
NATIVE: int
PTR: int
SHORT: int
LONGLONG: int
INT8: int
LITTLE_ENDIAN: int
LONG: int
UINT: int
ULONG: int
ULONGLONG: int
USHORT: int
UINT8: int
UINT16: int
UINT32: int
UINT64: int
INT64: int
BFUINT16: int
BFUINT32: int
BFUINT8: int
BFINT8: int
ARRAY: int
BFINT16: int
BFINT32: int
BF_LEN: int
INT: int
INT16: int
INT32: int
FLOAT64: int
BF_POS: int
BIG_ENDIAN: int
FLOAT32: int

def sizeof(struct, layout_type=NATIVE, /) -> int:
    """
    Return size of data structure in bytes. The *struct* argument can be
    either a structure class or a specific instantiated structure object
    (or its aggregate field).
    """
    ...

def bytes_at(addr, size) -> bytes:
    """
    Capture memory at the given address and size as bytes object. As bytes
    object is immutable, memory is actually duplicated and copied into
    bytes object, so if memory contents change later, created object
    retains original value.
    """
    ...

def bytearray_at(addr, size) -> bytearray:
    """
    Capture memory at the given address and size as bytearray object.
    Unlike bytes_at() function above, memory is captured by reference,
    so it can be both written too, and you will access current value
    at the given memory address.
    """
    ...

def addressof(obj) -> int:
    """
    Return address of an object. Argument should be bytes, bytearray or
    other object supporting buffer protocol (and address of this buffer
    is what actually returned).
    """
    ...

class struct:
    """
    Instantiate a "foreign data structure" object based on structure address in
    memory, descriptor (encoded as a dictionary), and layout type (see below).
    """

    def __init__(self, addr, descriptor, layout_type=NATIVE, /) -> None: ...



from _typeshed import Incomplete

_CONVERT: Incomplete
_RD_SCRATCH: Incomplete
_WR_SCRATCH: Incomplete

class DS18X20:
    ow: Incomplete
    buf: Incomplete
    def __init__(self, onewire) -> None: ...
    def scan(self): ...
    def convert_temp(self) -> None: ...
    def read_scratch(self, rom): ...
    def write_scratch(self, rom, buf) -> None: ...
    def read_temp(self, rom): ...



"""
Functions related to the ESP8266 and ESP32.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/esp.html

The ``esp`` module contains specific functions related to both the ESP8266 and
ESP32 modules.  Some functions are only available on one or the other of these
ports.
"""
from typing import Optional, Any
from _typeshed import Incomplete

LOG_NONE: int
LOG_WARNING: int
LOG_VERBOSE: int
LOG_DEBUG: int
LOG_INFO: int
LOG_ERROR: int

def osdebug(level) -> None:
    """
    Turn esp os debugging messages on or off.

    The *level* parameter sets the threshold for the log messages for all esp components.
    The log levels are defined as constants:

        * ``LOG_NONE`` -- No log output
        * ``LOG_ERROR`` -- Critical errors, software module can not recover on its own
        * ``LOG_WARN`` -- Error conditions from which recovery measures have been taken
        * ``LOG_INFO`` -- Information messages which describe normal flow of events
        * ``LOG_DEBUG`` -- Extra information which is not necessary for normal use (values, pointers, sizes, etc)
        * ``LOG_VERBOSE`` -- Bigger chunks of debugging information, or frequent messages
          which can potentially flood the output
    """
    ...

def flash_write(byte_offset, bytes) -> Incomplete: ...
def gpio_matrix_in(*args, **kwargs) -> Any: ...
def gpio_matrix_out(*args, **kwargs) -> Any: ...
def flash_user_start() -> Incomplete:
    """
    Read the memory offset at which the user flash space begins.
    """
    ...

def flash_erase(sector_no) -> Incomplete: ...
def flash_read(byte_offset, length_or_buffer) -> Incomplete: ...
def flash_size() -> Incomplete:
    """
    Read the total size of the flash memory.
    """
    ...



from . import simple as simple

class MQTTClient(simple.MQTTClient):
    DELAY: int
    DEBUG: bool
    def delay(self, i) -> None: ...
    def log(self, in_reconnect, e) -> None: ...
    def reconnect(self): ...
    def publish(self, topic, msg, retain: bool = ..., qos: int = ...): ...
    def wait_msg(self): ...
    def check_msg(self, attempts: int = ...): ...



"""
TLS/SSL wrapper for socket objects.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/ssl.html

CPython module: :mod:`python:ssl` https://docs.python.org/3/library/ssl.html .

This module provides access to Transport Layer Security (previously and
widely known as “Secure Sockets Layer”) encryption and peer authentication
facilities for network sockets, both client-side and server-side.
"""
from typing import IO, Any
from _typeshed import Incomplete
from stdlib.ssl import *

CERT_OPTIONAL: int
CERT_REQUIRED: int
CERT_NONE: int

def wrap_socket(
    sock, server_side=False, keyfile=None, certfile=None, cert_reqs=None, cadata=None, server_hostname=None, do_handshake=True
) -> IO:
    """
    Takes a `stream` *sock* (usually socket.socket instance of ``SOCK_STREAM`` type),
    and returns an instance of ssl.SSLSocket, which wraps the underlying stream in
    an SSL context. Returned object has the usual `stream` interface methods like
    ``read()``, ``write()``, etc.
    A server-side SSL socket should be created from a normal socket returned from
    :meth:`~socket.socket.accept()` on a non-SSL listening server socket.

    - *do_handshake* determines whether the handshake is done as part of the ``wrap_socket``
      or whether it is deferred to be done as part of the initial reads or writes
      (there is no ``do_handshake`` method as in CPython).
      For blocking sockets doing the handshake immediately is standard. For non-blocking
      sockets (i.e. when the *sock* passed into ``wrap_socket`` is in non-blocking mode)
      the handshake should generally be deferred because otherwise ``wrap_socket`` blocks
      until it completes. Note that in AXTLS the handshake can be deferred until the first
      read or write but it then blocks until completion.

    - *cert_reqs* determines whether the peer (server or client) must present a valid certificate.
      Note that for mbedtls based ports, ``ssl.CERT_NONE`` and ``ssl.CERT_OPTIONAL`` will not
      validate any certificate, only ``ssl.CERT_REQUIRED`` will.

    - *cadata* is a bytes object containing the CA certificate chain (in DER format) that will
      validate the peer's certificate.  Currently only a single DER-encoded certificate is supported.

    - *server_hostname* is for use as a client, and sets the hostname to check against the received
      server certificate.  It also sets the name for Server Name Indication (SNI), allowing the server
      to present the proper certificate.

    Depending on the underlying module implementation in a particular
    :term:`MicroPython port`, some or all keyword arguments above may be not supported.
    """
    ...



from _typeshed import Incomplete

listen_s: Incomplete
client_s: Incomplete
DEBUG: int
_DEFAULT_STATIC_HOST: Incomplete
static_host = _DEFAULT_STATIC_HOST

def server_handshake(cl): ...
def send_html(cl) -> None: ...
def setup_conn(port, accept_handler): ...
def accept_conn(listen_sock): ...
def stop() -> None: ...
def start(port: int = ..., password: Incomplete | None = ..., accept_handler=...) -> None: ...
def start_foreground(port: int = ..., password: Incomplete | None = ...) -> None: ...



"""
Collection and container types.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/collections.html

CPython module: :mod:`python:collections` https://docs.python.org/3/library/collections.html .

This module implements advanced collection and container types to
hold/accumulate various objects.
"""
from typing import Optional, Any
from stdlib.collections import OrderedDict as stdlib_OrderedDict, deque as stdlib_deque
from typing_extensions import NamedTuple as stdlib_NamedTuple
from _typeshed import Incomplete

def namedtuple(name, fields) -> stdlib_NamedTuple:
    """
    This is factory function to create a new namedtuple type with a specific
    name and set of fields. A namedtuple is a subclass of tuple which allows
    to access its fields not just by numeric index, but also with an attribute
    access syntax using symbolic field names. Fields is a sequence of strings
    specifying field names. For compatibility with CPython it can also be a
    a string with space-separated field named (but this is less efficient).
    Example of use::

        from collections import namedtuple

        MyTuple = namedtuple("MyTuple", ("id", "name"))
        t1 = MyTuple(1, "foo")
        t2 = MyTuple(2, "bar")
        print(t1.name)
        assert t2.name == t2[1]
    """
    ...

class OrderedDict(stdlib_OrderedDict):
    """
    ``dict`` type subclass which remembers and preserves the order of keys
    added. When ordered dict is iterated over, keys/items are returned in
    the order they were added::

        from collections import OrderedDict

        # To make benefit of ordered keys, OrderedDict should be initialized
        # from sequence of (key, value) pairs.
        d = OrderedDict([("z", 1), ("a", 2)])
        # More items can be added as usual
        d["w"] = 5
        d["b"] = 3
        for k, v in d.items():
            print(k, v)

    Output::

        z 1
        a 2
        w 5
        b 3
    """

    def popitem(self, *args, **kwargs) -> Any: ...
    def pop(self, *args, **kwargs) -> Any: ...
    def values(self, *args, **kwargs) -> Any: ...
    def setdefault(self, *args, **kwargs) -> Any: ...
    def update(self, *args, **kwargs) -> Any: ...
    def copy(self, *args, **kwargs) -> Any: ...
    def clear(self, *args, **kwargs) -> Any: ...
    def keys(self, *args, **kwargs) -> Any: ...
    def get(self, *args, **kwargs) -> Any: ...
    def items(self, *args, **kwargs) -> Any: ...
    @classmethod
    def fromkeys(cls, *args, **kwargs) -> Any: ...
    def __init__(self, *args, **kwargs) -> None: ...

class deque(stdlib_deque):
    """
    Deques (double-ended queues) are a list-like container that support O(1)
    appends and pops from either side of the deque.  New deques are created
    using the following arguments:

        - *iterable* must be the empty tuple, and the new deque is created empty.

        - *maxlen* must be specified and the deque will be bounded to this
          maximum length.  Once the deque is full, any new items added will
          discard items from the opposite end.

        - The optional *flags* can be 1 to check for overflow when adding items.

    As well as supporting `bool` and `len`, deque objects have the following
    methods:
    """

    def popleft(self) -> Incomplete:
        """
        Remove and return an item from the left side of the deque.
        Raises IndexError if no items are present.
        """
        ...
    def append(self, x) -> Incomplete:
        """
        Add *x* to the right side of the deque.
        Raises IndexError if overflow checking is enabled and there is no more room left.
        """
        ...
    def __init__(self, iterable, maxlen, flags: Optional[Any] = None) -> None: ...



"""
Hashing algorithms.

MicroPython module: https://docs.micropython.org/en/v1.20.0/library/hashlib.html

CPython module: :mod:`python:hashlib` https://docs.python.org/3/library/hashlib.html .

This module implements binary data hashing algorithms. The exact inventory
of available algorithms depends on a board. Among the algorithms which may
be implemented:

* SHA256 - The current generation, modern hashing algorithm (of SHA2 series).
  It is suitable for cryptographically-secure purposes. Included in the
  MicroPython core and any board is recommended to provide this, unless
  it has particular code size constraints.

* SHA1 - A previous generation algorithm. Not recommended for new usages,
  but SHA1 is a part of number of Internet standards and existing
  applications, so boards targeting network connectivity and
  interoperability will try to provide this.

* MD5 - A legacy algorithm, not considered cryptographically secure. Only
  selected boards, targeting interoperability with legacy applications,
  will offer this.
"""
from typing import Optional, Any
from _typeshed import Incomplete

class sha256:
    """
    Create an SHA256 hasher object and optionally feed ``data`` into it.
    """

    def digest(self, *args, **kwargs) -> Any: ...
    def update(self, *args, **kwargs) -> Any: ...
    def __init__(self, data: Optional[Any] = None) -> None: ...

class sha1:
    """
    Create an SHA1 hasher object and optionally feed ``data`` into it.
    """

    def digest(self, *args, **kwargs) -> Any: ...
    def update(self, *args, **kwargs) -> Any: ...
    def __init__(self, data: Optional[Any] = None) -> None: ...



from . import core as core
from _typeshed import Incomplete
from collections.abc import Generator

class Stream:
    s: Incomplete
    e: Incomplete
    out_buf: bytes
    def __init__(self, s, e=...) -> None: ...
    def get_extra_info(self, v): ...
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc, tb) -> None: ...
    def close(self) -> None: ...
    async def wait_closed(self) -> None: ...
    def read(self, n: int = ...) -> Generator[Incomplete, None, Incomplete]: ...
    def readinto(self, buf) -> Generator[Incomplete, None, Incomplete]: ...
    def readexactly(self, n) -> Generator[Incomplete, None, Incomplete]: ...
    def readline(self) -> Generator[Incomplete, None, Incomplete]: ...
    def write(self, buf) -> None: ...
    def drain(self) -> Generator[Incomplete, Incomplete, Incomplete]: ...

StreamReader = Stream
StreamWriter = Stream

def open_connection(host, port) -> Generator[Incomplete, None, Incomplete]: ...

class Server:
    async def __aenter__(self): ...
    async def __aexit__(self, exc_type, exc, tb) -> None: ...
    def close(self) -> None: ...
    async def wait_closed(self) -> None: ...
    async def _serve(self, s, cb) -> Generator[Incomplete, None, None]: ...

async def start_server(cb, host, port, backlog: int = ...): ...
async def stream_awrite(self, buf, off: int = ..., sz: int = ...) -> None: ...



from typing import Any

class websocket:
    def readline(self, *args, **kwargs) -> Any: ...
    def ioctl(self, *args, **kwargs) -> Any: ...
    def write(self, *args, **kwargs) -> Any: ...
    def close(self, *args, **kwargs) -> Any: ...
    def readinto(self, *args, **kwargs) -> Any: ...
    def read(self, *args, **kwargs) -> Any: ...
    def __init__(self, *argv, **kwargs) -> None: ...



from _typeshed import Incomplete

_PACKAGE_INDEX: Incomplete
_CHUNK_SIZE: int

def _ensure_path_exists(path) -> None: ...
def _chunk(src, dest) -> None: ...
def _check_exists(path, short_hash): ...
def _rewrite_url(url, branch: Incomplete | None = ...): ...
def _download_file(url, dest): ...
def _install_json(package_json_url, index, target, version, mpy): ...
def _install_package(package, index, target, version, mpy): ...
def install(
    package, index: Incomplete | None = ..., target: Incomplete | None = ..., version: Incomplete | None = ..., mpy: bool = ...
) -> None: ...
